@@ -21,8 +21,8 @@ package cdata
 
 // implement handling of the Arrow C Data Interface. At least from a consuming side.
 
-// #include "arrow/c/abi.h"
-// #include "arrow/c/helpers.h"
+// #include "abi.h"
+// #include "helpers.h"
 // #include <stdlib.h>
 // int stream_get_schema(struct ArrowArrayStream* st, struct ArrowSchema* out) { return st->get_schema(st, out); }
 // int stream_get_next(struct ArrowArrayStream* st, struct ArrowArray* out) { return st->get_next(st, out); }
@@ -64,6 +64,11 @@ type (
 	CArrowArray = C.struct_ArrowArray
 	// CArrowArrayStream is the C Stream Interface object for handling streams of record batches.
 	CArrowArrayStream = C.struct_ArrowArrayStream
+
+	CArrowAsyncDeviceStreamHandler = C.struct_ArrowAsyncDeviceStreamHandler
+	CArrowAsyncProducer            = C.struct_ArrowAsyncProducer
+	CArrowAsyncTask                = C.struct_ArrowAsyncTask
+	CArrowDeviceArray              = C.struct_ArrowDeviceArray
 )
 
 // Map from the defined strings to their corresponding arrow.DataType interface
@@ -189,7 +194,8 @@ func importSchema(schema *CArrowSchema) (ret arrow.Field, err error) {
 			ret.Type = &arrow.DictionaryType{
 				IndexType: ret.Type,
 				ValueType: valueField.Type,
-				Ordered:   schema.dictionary.flags&C.ARROW_FLAG_DICTIONARY_ORDERED != 0}
+				Ordered:   schema.dictionary.flags&C.ARROW_FLAG_DICTIONARY_ORDERED != 0,
+			}
 		}
 
 		return
@@ -455,7 +461,7 @@ func (imp *cimporter) doImportArr(src *CArrowArray) error {
 	// struct immediately after import, since we have no imported
 	// memory that we have to track the lifetime of.
 	defer func() {
-		if imp.alloc.bufCount == 0 {
+		if imp.alloc.bufCount.Load() == 0 {
 			C.ArrowArrayRelease(imp.arr)
 			C.free(unsafe.Pointer(imp.arr))
 		}
@@ -657,9 +663,7 @@ func (imp *cimporter) importStringLike(offsetByteWidth int64) (err error) {
 		return
 	}
 
-	var (
-		nulls, offsets, values *memory.Buffer
-	)
+	var nulls, offsets, values *memory.Buffer
 	if nulls, err = imp.importNullBitmap(0); err != nil {
 		return
 	}
