@@ -32,9 +32,9 @@ import (
 	"github.com/apache/arrow-go/v18/arrow/internal/debug"
 	"github.com/apache/arrow-go/v18/arrow/memory"
 	"github.com/apache/arrow-go/v18/arrow/scalar"
-	"github.com/substrait-io/substrait-go/expr"
-	"github.com/substrait-io/substrait-go/extensions"
-	"github.com/substrait-io/substrait-go/types"
+	"github.com/substrait-io/substrait-go/v3/expr"
+	"github.com/substrait-io/substrait-go/v3/extensions"
+	"github.com/substrait-io/substrait-go/v3/types"
 )
 
 func makeExecBatch(ctx context.Context, schema *arrow.Schema, partial compute.Datum) (out compute.ExecBatch, err error) {
@@ -173,7 +173,7 @@ func GetExtensionIDSet(ctx context.Context) ExtensionIDSet {
 	v, ok := ctx.Value(extCtxKey{}).(ExtensionIDSet)
 	if !ok {
 		return NewExtensionSet(
-			expr.NewEmptyExtensionRegistry(&extensions.DefaultCollection),
+			expr.NewEmptyExtensionRegistry(extensions.GetDefaultCollectionWithNoError()),
 			GetExtensionRegistry(ctx))
 	}
 	return v
@@ -434,7 +434,8 @@ func ExecuteScalarSubstrait(ctx context.Context, expression *expr.Extended, part
 	}
 
 	reg := GetExtensionRegistry(ctx)
-	set := NewExtensionSet(expr.NewExtensionRegistry(expression.Extensions, &extensions.DefaultCollection), reg)
+	set := NewExtensionSet(expr.NewExtensionRegistry(expression.Extensions,
+		extensions.GetDefaultCollectionWithNoError()), reg)
 	sc, err := ToArrowSchema(expression.BaseSchema, set)
 	if err != nil {
 		return nil, err
@@ -523,7 +524,6 @@ func executeScalarBatch(ctx context.Context, input compute.ExecBatch, exp expr.E
 			err       error
 			allScalar = true
 			args      = make([]compute.Datum, e.NArgs())
-			argTypes  = make([]arrow.DataType, e.NArgs())
 		)
 		for i := 0; i < e.NArgs(); i++ {
 			switch v := e.Arg(i).(type) {
@@ -542,20 +542,23 @@ func executeScalarBatch(ctx context.Context, input compute.ExecBatch, exp expr.E
 			default:
 				return nil, arrow.ErrNotImplemented
 			}
-
-			argTypes[i] = args[i].(compute.ArrayLikeDatum).Type()
 		}
 
 		_, conv, ok := ext.DecodeFunction(e.FuncRef())
 		if !ok {
-			return nil, arrow.ErrNotImplemented
+			return nil, fmt.Errorf("%w: %s", arrow.ErrNotImplemented, e.Name())
 		}
 
-		fname, opts, err := conv(e)
+		fname, args, opts, err := conv(e, args)
 		if err != nil {
 			return nil, err
 		}
 
+		argTypes := make([]arrow.DataType, len(args))
+		for i, arg := range args {
+			argTypes[i] = arg.(compute.ArrayLikeDatum).Type()
+		}
+
 		ectx := compute.GetExecCtx(ctx)
 		fn, ok := ectx.Registry.GetFunction(fname)
 		if !ok {
@@ -571,6 +574,25 @@ func executeScalarBatch(ctx context.Context, input compute.ExecBatch, exp expr.E
 			return nil, err
 		}
 
+		var newArgs []compute.Datum
+		// cast arguments if necessary
+		for i, arg := range args {
+			if !arrow.TypeEqual(argTypes[i], arg.(compute.ArrayLikeDatum).Type()) {
+				if newArgs == nil {
+					newArgs = make([]compute.Datum, len(args))
+					copy(newArgs, args)
+				}
+				newArgs[i], err = compute.CastDatum(ctx, arg, compute.SafeCastOptions(argTypes[i]))
+				if err != nil {
+					return nil, err
+				}
+				defer newArgs[i].Release()
+			}
+		}
+		if newArgs != nil {
+			args = newArgs
+		}
+
 		kctx := &exec.KernelCtx{Ctx: ctx, Kernel: k}
 		init := k.GetInitFn()
 		kinitArgs := exec.KernelInitArgs{Kernel: k, Inputs: argTypes, Options: opts}
@@ -611,9 +633,10 @@ func executeScalarBatch(ctx context.Context, input compute.ExecBatch, exp expr.E
 
 		if ctx.Err() == context.Canceled && result != nil {
 			result.Release()
+			result = nil
 		}
 
-		return result, nil
+		return result, err
 	}
 
 	return nil, arrow.ErrNotImplemented
