@@ -18,6 +18,7 @@ package pqarrow
 
 import (
 	"encoding/base64"
+	"errors"
 	"fmt"
 	"math"
 	"strconv"
@@ -240,9 +241,26 @@ func repFromNullable(isnullable bool) parquet.Repetition {
 	return parquet.Repetitions.Required
 }
 
-func structToNode(typ *arrow.StructType, name string, nullable bool, props *parquet.WriterProperties, arrprops ArrowWriterProperties) (schema.Node, error) {
+func variantToNode(t *variantExtensionType, field arrow.Field, props *parquet.WriterProperties, arrProps ArrowWriterProperties) (schema.Node, error) {
+	metadataNode, err := fieldToNode("metadata", t.Metadata(), props, arrProps)
+	if err != nil {
+		return nil, err
+	}
+
+	valueNode, err := fieldToNode("value", t.Value(), props, arrProps)
+	if err != nil {
+		return nil, err
+	}
+
+	return schema.NewGroupNodeLogical(field.Name, repFromNullable(field.Nullable),
+		schema.FieldList{metadataNode, valueNode}, schema.VariantLogicalType{},
+		fieldIDFromMeta(field.Metadata))
+}
+
+func structToNode(field arrow.Field, props *parquet.WriterProperties, arrprops ArrowWriterProperties) (schema.Node, error) {
+	typ := field.Type.(*arrow.StructType)
 	if typ.NumFields() == 0 {
-		return nil, fmt.Errorf("cannot write struct type '%s' with no children field to parquet. Consider adding a dummy child", name)
+		return nil, fmt.Errorf("cannot write struct type '%s' with no children field to parquet. Consider adding a dummy child", field.Name)
 	}
 
 	children := make(schema.FieldList, 0, typ.NumFields())
@@ -254,7 +272,7 @@ func structToNode(typ *arrow.StructType, name string, nullable bool, props *parq
 		children = append(children, n)
 	}
 
-	return schema.NewGroupNode(name, repFromNullable(nullable), children, -1)
+	return schema.NewGroupNode(field.Name, repFromNullable(field.Nullable), children, fieldIDFromMeta(field.Metadata))
 }
 
 func fieldToNode(name string, field arrow.Field, props *parquet.WriterProperties, arrprops ArrowWriterProperties) (schema.Node, error) {
@@ -267,21 +285,16 @@ func fieldToNode(name string, field arrow.Field, props *parquet.WriterProperties
 			return nil, xerrors.New("nulltype arrow field must be nullable")
 		}
 	case arrow.STRUCT:
-		return structToNode(field.Type.(*arrow.StructType), field.Name, field.Nullable, props, arrprops)
+		return structToNode(field, props, arrprops)
 	case arrow.FIXED_SIZE_LIST, arrow.LIST:
-		var elem arrow.DataType
-		if lt, ok := field.Type.(*arrow.ListType); ok {
-			elem = lt.Elem()
-		} else {
-			elem = field.Type.(*arrow.FixedSizeListType).Elem()
-		}
+		elemField := field.Type.(arrow.ListLikeType).ElemField()
 
-		child, err := fieldToNode(name, arrow.Field{Name: name, Type: elem, Nullable: true}, props, arrprops)
+		child, err := fieldToNode(name, elemField, props, arrprops)
 		if err != nil {
 			return nil, err
 		}
 
-		return schema.ListOf(child, repFromNullable(field.Nullable), -1)
+		return schema.ListOfWithName(name, child, repFromNullable(field.Nullable), fieldIDFromMeta(field.Metadata))
 	case arrow.DICTIONARY:
 		// parquet has no dictionary type, dictionary is encoding, not schema level
 		dictType := field.Type.(*arrow.DictionaryType)
@@ -307,9 +320,14 @@ func fieldToNode(name string, field arrow.Field, props *parquet.WriterProperties
 			}
 			return schema.NewGroupNode(field.Name, repFromNullable(field.Nullable), schema.FieldList{
 				keyvalNode,
-			}, -1)
+			}, fieldIDFromMeta(field.Metadata))
+		}
+		return schema.MapOf(field.Name, keyNode, valueNode, repFromNullable(field.Nullable), fieldIDFromMeta(field.Metadata))
+	case arrow.EXTENSION:
+		extType := field.Type.(arrow.ExtensionType)
+		if extType.ExtensionName() == "parquet.variant" {
+			return variantToNode(extType.(*variantExtensionType), field, props, arrprops)
 		}
-		return schema.MapOf(field.Name, keyNode, valueNode, repFromNullable(field.Nullable), -1)
 	}
 
 	// Not a GroupNode
@@ -514,8 +532,14 @@ func arrowFromFLBA(logical schema.LogicalType, length int) (arrow.DataType, erro
 	switch logtype := logical.(type) {
 	case schema.DecimalLogicalType:
 		return arrowDecimal(logtype), nil
-	case schema.NoLogicalType, schema.IntervalLogicalType, schema.UUIDLogicalType:
+	case schema.NoLogicalType, schema.IntervalLogicalType:
 		return &arrow.FixedSizeBinaryType{ByteWidth: int(length)}, nil
+	case schema.UUIDLogicalType:
+		uuidType := arrow.GetExtensionType("arrow.uuid")
+		if uuidType == nil {
+			return &arrow.FixedSizeBinaryType{ByteWidth: int(length)}, nil
+		}
+		return uuidType, nil
 	case schema.Float16LogicalType:
 		return &arrow.Float16Type{}, nil
 	default:
@@ -690,7 +714,7 @@ func listToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *s
 				return err
 			}
 		} else {
-			if err := groupToStructField(listGroup, currentLevels, ctx, out, &out.Children[0]); err != nil {
+			if err := groupToStructField(listGroup, currentLevels, ctx, &out.Children[0]); err != nil {
 				return err
 			}
 		}
@@ -711,12 +735,20 @@ func listToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *s
 			arrowType = &arrow.DictionaryType{IndexType: arrow.PrimitiveTypes.Int32, ValueType: arrowType}
 		}
 
+		if arrow.IsBinaryLike(arrowType.ID()) && ctx.props.ForceLarge(colIndex) {
+			switch arrowType.ID() {
+			case arrow.STRING:
+				arrowType = arrow.BinaryTypes.LargeString
+			case arrow.BINARY:
+				arrowType = arrow.BinaryTypes.LargeBinary
+			}
+		}
+
 		itemField := arrow.Field{Name: listNode.Name(), Type: arrowType, Nullable: false, Metadata: createFieldMeta(int(listNode.FieldID()))}
 		populateLeaf(colIndex, &itemField, currentLevels, ctx, out, &out.Children[0])
 	}
 
-	out.Field = &arrow.Field{Name: n.Name(), Type: arrow.ListOfField(
-		arrow.Field{Name: listNode.Name(), Type: out.Children[0].Field.Type, Nullable: true}),
+	out.Field = &arrow.Field{Name: n.Name(), Type: arrow.ListOfField(*out.Children[0].Field),
 		Nullable: n.RepetitionType() == parquet.Repetitions.Optional, Metadata: createFieldMeta(int(n.FieldID()))}
 
 	out.LevelInfo = currentLevels
@@ -726,7 +758,7 @@ func listToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *s
 	return nil
 }
 
-func groupToStructField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *schemaTree, parent, out *SchemaField) error {
+func groupToStructField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *schemaTree, out *SchemaField) error {
 	arrowFields := make([]arrow.Field, 0, n.NumFields())
 	out.Children = make([]SchemaField, n.NumFields())
 
@@ -738,7 +770,7 @@ func groupToStructField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *
 	}
 
 	out.Field = &arrow.Field{Name: n.Name(), Type: arrow.StructOf(arrowFields...),
-		Nullable: n.RepetitionType() == parquet.Repetitions.Optional, Metadata: createFieldMeta(int(n.FieldID()))}
+		Nullable: n.RepetitionType() != parquet.Repetitions.Required, Metadata: createFieldMeta(int(n.FieldID()))}
 	out.LevelInfo = currentLevels
 	return nil
 }
@@ -811,7 +843,7 @@ func mapToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *sc
 		Nullable: false, Metadata: createFieldMeta(int(kvgroup.FieldID()))}
 
 	kvfield.LevelInfo = currentLevels
-	out.Field = &arrow.Field{Name: n.Name(), Type: arrow.MapOf(keyField.Field.Type, valueField.Field.Type),
+	out.Field = &arrow.Field{Name: n.Name(), Type: arrow.MapOfFields(*keyField.Field, *valueField.Field),
 		Nullable: n.RepetitionType() == parquet.Repetitions.Optional,
 		Metadata: createFieldMeta(int(n.FieldID()))}
 	out.LevelInfo = currentLevels
@@ -821,11 +853,29 @@ func mapToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *sc
 	return nil
 }
 
+func variantToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *schemaTree, parent, out *SchemaField) error {
+	// this is for unshredded variants. shredded variants may have more fields
+	if n.NumFields() != 2 {
+		return errors.New("VARIANT group must have exactly 2 children")
+	}
+
+	var err error
+	if err = groupToStructField(n, currentLevels, ctx, out); err != nil {
+		return err
+	}
+
+	storageType := out.Field.Type
+	out.Field.Type, err = newVariantType(storageType)
+	return err
+}
+
 func groupToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *schemaTree, parent, out *SchemaField) error {
 	if n.LogicalType().Equals(schema.NewListLogicalType()) {
 		return listToSchemaField(n, currentLevels, ctx, parent, out)
 	} else if n.LogicalType().Equals(schema.MapLogicalType{}) {
 		return mapToSchemaField(n, currentLevels, ctx, parent, out)
+	} else if n.LogicalType().Equals(schema.VariantLogicalType{}) {
+		return variantToSchemaField(n, currentLevels, ctx, parent, out)
 	}
 
 	if n.RepetitionType() == parquet.Repetitions.Repeated {
@@ -837,7 +887,7 @@ func groupToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *
 		// }
 		out.Children = make([]SchemaField, 1)
 		repeatedAncestorDef := currentLevels.IncrementRepeated()
-		if err := groupToStructField(n, currentLevels, ctx, out, &out.Children[0]); err != nil {
+		if err := groupToStructField(n, currentLevels, ctx, &out.Children[0]); err != nil {
 			return err
 		}
 
@@ -850,7 +900,7 @@ func groupToSchemaField(n *schema.GroupNode, currentLevels file.LevelInfo, ctx *
 	}
 
 	currentLevels.Increment(n)
-	return groupToStructField(n, currentLevels, ctx, parent, out)
+	return groupToStructField(n, currentLevels, ctx, out)
 }
 
 func createFieldMeta(fieldID int) arrow.Metadata {
@@ -885,6 +935,15 @@ func nodeToSchemaField(n schema.Node, currentLevels file.LevelInfo, ctx *schemaT
 		arrowType = &arrow.DictionaryType{IndexType: arrow.PrimitiveTypes.Int32, ValueType: arrowType}
 	}
 
+	if arrow.IsBinaryLike(arrowType.ID()) && ctx.props.ForceLarge(colIndex) {
+		switch arrowType.ID() {
+		case arrow.STRING:
+			arrowType = arrow.BinaryTypes.LargeString
+		case arrow.BINARY:
+			arrowType = arrow.BinaryTypes.LargeBinary
+		}
+	}
+
 	if primitive.RepetitionType() == parquet.Repetitions.Repeated {
 		// one-level list encoding e.g. a: repeated int32;
 		repeatedAncestorDefLevel := currentLevels.IncrementRepeated()
@@ -984,13 +1043,14 @@ func applyOriginalStorageMetadata(origin arrow.Field, inferred *SchemaField) (mo
 			return
 		}
 
-		if !arrow.TypeEqual(extType.StorageType(), inferred.Field.Type) {
-			return modified, fmt.Errorf("%w: mismatch storage type '%s' for extension type '%s'",
-				arrow.ErrInvalid, inferred.Field.Type, extType)
-		}
+		if modified && !arrow.TypeEqual(extType, inferred.Field.Type) {
+			if !arrow.TypeEqual(extType.StorageType(), inferred.Field.Type) {
+				return modified, fmt.Errorf("%w: mismatch storage type '%s' for extension type '%s'",
+					arrow.ErrInvalid, inferred.Field.Type, extType)
+			}
 
-		inferred.Field.Type = extType
-		modified = true
+			inferred.Field.Type = extType
+		}
 	case arrow.SPARSE_UNION, arrow.DENSE_UNION:
 		err = xerrors.New("unimplemented type")
 	case arrow.STRUCT:
