@@ -292,7 +292,7 @@ func TestConvertArrowFloat16(t *testing.T) {
 	}
 }
 
-func TestCoerceTImestampV1(t *testing.T) {
+func TestCoerceTimestampV1(t *testing.T) {
 	parquetFields := make(schema.FieldList, 0)
 	arrowFields := make([]arrow.Field, 0)
 
@@ -311,7 +311,7 @@ func TestCoerceTImestampV1(t *testing.T) {
 	}
 }
 
-func TestAutoCoerceTImestampV1(t *testing.T) {
+func TestAutoCoerceTimestampV1(t *testing.T) {
 	parquetFields := make(schema.FieldList, 0)
 	arrowFields := make([]arrow.Field, 0)
 
@@ -370,11 +370,11 @@ func TestListStructBackwardCompatible(t *testing.T) {
 	// the 3-level encoding. Schema looks like:
 	//
 	//     required group field_id=-1 root {
-	//       optional group field_id=-1 answers (List) {
+	//       optional group field_id=1 answers (List) {
 	//		   repeated group field_id=-1 array {
-	//           optional byte_array field_id=-1 type (String);
-	//           optional byte_array field_id=-1 rdata (String);
-	//           optional byte_array field_id=-1 class (String);
+	//           optional byte_array field_id=2 type (String);
+	//           optional byte_array field_id=3 rdata (String);
+	//           optional byte_array field_id=4 class (String);
 	//         }
 	//       }
 	//     }
@@ -382,12 +382,12 @@ func TestListStructBackwardCompatible(t *testing.T) {
 	// Instead of the proper 3-level encoding which would be:
 	//
 	//     repeated group field_id=-1 schema {
-	//       optional group field_id=-1 answers (List) {
+	//       optional group field_id=1 answers (List) {
 	//         repeated group field_id=-1 list {
 	//           optional group field_id=-1 element {
-	//             optional byte_array field_id=-1 type (String);
-	//             optional byte_array field_id=-1 rdata (String);
-	//             optional byte_array field_id=-1 class (String);
+	//             optional byte_array field_id=2 type (String);
+	//             optional byte_array field_id=3 rdata (String);
+	//             optional byte_array field_id=4 class (String);
 	//           }
 	//         }
 	//       }
@@ -397,30 +397,33 @@ func TestListStructBackwardCompatible(t *testing.T) {
 		schema.Must(schema.NewGroupNodeLogical("answers", parquet.Repetitions.Optional, schema.FieldList{
 			schema.Must(schema.NewGroupNode("array", parquet.Repetitions.Repeated, schema.FieldList{
 				schema.MustPrimitive(schema.NewPrimitiveNodeLogical("type", parquet.Repetitions.Optional,
-					schema.StringLogicalType{}, parquet.Types.ByteArray, -1, -1)),
+					schema.StringLogicalType{}, parquet.Types.ByteArray, -1, 2)),
 				schema.MustPrimitive(schema.NewPrimitiveNodeLogical("rdata", parquet.Repetitions.Optional,
-					schema.StringLogicalType{}, parquet.Types.ByteArray, -1, -1)),
+					schema.StringLogicalType{}, parquet.Types.ByteArray, -1, 3)),
 				schema.MustPrimitive(schema.NewPrimitiveNodeLogical("class", parquet.Repetitions.Optional,
-					schema.StringLogicalType{}, parquet.Types.ByteArray, -1, -1)),
-			}, -1)),
-		}, schema.NewListLogicalType(), -1)),
+					schema.StringLogicalType{}, parquet.Types.ByteArray, -1, 4)),
+			}, 5)),
+		}, schema.NewListLogicalType(), 1)),
 	}, -1)))
 
-	meta := arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"-1"})
 	// desired equivalent arrow schema would be list<item: struct<type: utf8, rdata: utf8, class: utf8>>
 	arrowSchema := arrow.NewSchema(
 		[]arrow.Field{
 			{Name: "answers", Type: arrow.ListOfField(arrow.Field{
 				Name: "array", Type: arrow.StructOf(
-					arrow.Field{Name: "type", Type: arrow.BinaryTypes.String, Nullable: true, Metadata: meta},
-					arrow.Field{Name: "rdata", Type: arrow.BinaryTypes.String, Nullable: true, Metadata: meta},
-					arrow.Field{Name: "class", Type: arrow.BinaryTypes.String, Nullable: true, Metadata: meta},
-				), Nullable: true}), Nullable: true, Metadata: meta},
+					arrow.Field{Name: "type", Type: arrow.BinaryTypes.String, Nullable: true,
+						Metadata: arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"2"})},
+					arrow.Field{Name: "rdata", Type: arrow.BinaryTypes.String, Nullable: true,
+						Metadata: arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"3"})},
+					arrow.Field{Name: "class", Type: arrow.BinaryTypes.String, Nullable: true,
+						Metadata: arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"4"})},
+				), Nullable: true, Metadata: arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"5"})}),
+				Nullable: true, Metadata: arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"1"})},
 		}, nil)
 
 	arrsc, err := pqarrow.FromParquet(pqSchema, nil, metadata.KeyValueMetadata{})
 	assert.NoError(t, err)
-	assert.True(t, arrowSchema.Equal(arrsc))
+	assert.True(t, arrowSchema.Equal(arrsc), arrowSchema.String(), arrsc.String())
 }
 
 // TestUnsupportedTypes tests the error message for unsupported types. This test should be updated
@@ -448,3 +451,56 @@ func TestUnsupportedTypes(t *testing.T) {
 		})
 	}
 }
+
+func TestProperListElementNullability(t *testing.T) {
+	arrSchema := arrow.NewSchema([]arrow.Field{
+		{Name: "qux", Type: arrow.ListOfField(
+			arrow.Field{
+				Name:     "element",
+				Type:     arrow.BinaryTypes.String,
+				Nullable: false,
+				Metadata: arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"-1"})}),
+			Nullable: false, Metadata: arrow.NewMetadata([]string{"PARQUET:field_id"}, []string{"-1"})},
+	}, nil)
+
+	pqSchema, err := pqarrow.ToParquet(arrSchema, nil, pqarrow.DefaultWriterProps())
+	require.NoError(t, err)
+	assert.EqualValues(t, 1, pqSchema.Column(0).MaxDefinitionLevel())
+	assert.Equal(t, parquet.Repetitions.Required, pqSchema.Column(0).SchemaNode().RepetitionType())
+
+	outSchema, err := pqarrow.FromParquet(pqSchema, nil, metadata.KeyValueMetadata{})
+	require.NoError(t, err)
+	assert.True(t, arrSchema.Equal(outSchema), "expected: %s, got: %s", arrSchema, outSchema)
+}
+
+func TestConvertSchemaParquetVariant(t *testing.T) {
+	// unshredded variant:
+	// optional group variant_col {
+	//   required binary metadata;
+	//   required binary value;
+	// }
+	//
+	// shredded variants will be added later
+	parquetFields := make(schema.FieldList, 0)
+	metadata := schema.NewByteArrayNode("metadata", parquet.Repetitions.Required, -1)
+	value := schema.NewByteArrayNode("value", parquet.Repetitions.Required, -1)
+
+	variant, err := schema.NewGroupNodeLogical("variant_unshredded", parquet.Repetitions.Optional,
+		schema.FieldList{metadata, value}, schema.VariantLogicalType{}, -1)
+	require.NoError(t, err)
+	parquetFields = append(parquetFields, variant)
+
+	pqschema := schema.NewSchema(schema.MustGroup(schema.NewGroupNode("schema", parquet.Repetitions.Required, parquetFields, -1)))
+	outSchema, err := pqarrow.FromParquet(pqschema, nil, nil)
+	require.NoError(t, err)
+
+	assert.EqualValues(t, 1, outSchema.NumFields())
+	assert.Equal(t, "variant_unshredded", outSchema.Field(0).Name)
+	assert.Equal(t, arrow.EXTENSION, outSchema.Field(0).Type.ID())
+
+	assert.Equal(t, "parquet.variant", outSchema.Field(0).Type.(arrow.ExtensionType).ExtensionName())
+
+	sc, err := pqarrow.ToParquet(outSchema, nil, pqarrow.DefaultWriterProps())
+	require.NoError(t, err)
+	assert.True(t, pqschema.Equals(sc), pqschema.String(), sc.String())
+}
